#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <cmath>
#include <csignal>
#include <ctype.h>
#include <unistd.h>
#include <termio.h>
#include <sys/ioctl.h>
#include <string>
#include <vector>

/* Version 1.00 - Copyright (C) 1992,94,99 Bisqwit (http://iki.fi/bisqwit/) */

static int Quittah=0;
/* These aren't ansi and so won't
 * pass -ansi without being declared.
 */
extern "C" int ioctl(int,int,...);
extern "C" void usleep(unsigned long);

#define voitto 65535U

static const unsigned char XOZGE[] = "XOZGE";
static const unsigned char Attrs[] = {9,10,12,13,14};

static const double rnd()
{
	return rand() / (1+(double)RAND_MAX);
}
static const unsigned Random(unsigned n)
{
	return unsigned(n*rnd());
}

static const string Str(int count, const unsigned char n)
{
	string s;
	while(count) s += n, count--;
	return s;
}

static int COLS=80, LINES=25;

static class Video
{
	int wx,wy;
	unsigned char TextAttr;
private:
	void ShowCurs(const bool c)
	{
		cout << (c ? "\33[?25h" : "\33[?25l");
	}
	void GetScreenGeometry()
	{
	#if defined(DJGPP) || defined(__BORLANDC__)
	    struct text_info w;
	    gettextinfo(&w);
	    LINES=w.screenheight;
	    COLS =w.screenwidth;
	#else
	#ifdef TIOCGWINSZ
	    struct winsize w;
	    if(ioctl(STDOUT_FILENO, TIOCGWINSZ, &w) >= 0)
	    {
	        LINES=w.ws_row;
	        COLS =w.ws_col;
	    }
	#else
	#ifdef WIOCGETD
	    struct uwdata w;
	    if(ioctl(STDOUT_FILENO, WIOCGETD, &w) >= 0)
	    {
	        LINES = w.uw_height / w.uw_vs;
	        COLS  = w.uw_width / w.uw_hs;
	    }
	#endif
	#endif
	#endif
	}
	const unsigned char SetAttr(unsigned char n)
	{
		unsigned char m[]="04261537";
		cout << "\33[0;";
		if(n&8)cout << "1;";
		if(n&128)cout << "5;";
		n&=127, cout << '3' << m[n&7] << ";4" << m[n>>4] << 'm';
		return(TextAttr=n);
	}
	struct termio back;
public:
	Video()
	{
		ioctl(0, TCGETA, &back);
		GetScreenGeometry();
		SetAttr(7);
		clear();
		cout << "\33(U";
	}
	~Video()
	{
		ioctl(0, TCSETA, &back);	/* restore original terminal mode	*/
		SetAttr(7);
		Goto(LINES, 1);
		cout << "\n\n\33(B\33[?2c";
		ShowCurs(1);
	}
	Video &operator=(const unsigned char attr)
	{
		SetAttr(attr);
		return *this;
	}
	inline Video &operator << (const string &s)
	{
		return *this << s.c_str();
	}
	Video &operator << (const int a)
	{ char Buf[10]; sprintf(Buf, "%d", a); return *this << Buf; }
	Video &operator << (const unsigned a)
	{ char Buf[10]; sprintf(Buf, "%u", a); return *this << Buf; }
	Video &operator << (const char *s)
	{
		while(*s)*this << *s++;
		return *this;
	}
	inline Video &operator << (const unsigned char c)
	{
		return *this << (const char)c;
	}
	Video &operator << (const char c)
	{
		if(c=='\n')wx=1, wy++;
		else if(c!='\b')wx++;
		else if(wx>1)wx--;
		cout << c;
		return *this;
	}	
	inline void clear()
	{
		cout << "\33[2J";
		wx=wy=1;
	}
	inline void Flush() { cout << flush; }
	const unsigned char getkey(int wait=0)
	{
		struct termio term = back;
		term.c_lflag &= ~ECHO & ~ICANON;
		term.c_cc[VMIN] = wait;		/* when set to 0, getch() will return	*/
		ioctl(0, TCSETA, &term);	/* immediately, if no key was pressed	*/
									/* (return value is then EOF)		*/
		int c = getchar();
		if(c!=EOF||Quittah)return(unsigned char)c;
		Flush();
		usleep(20000);
		return 0;
	}
	inline const int &wherey() { return wy; }
	inline const int &wherex() { return wx; }
	Video &Goto(const int y,const int x,const int v=-1)
	{
		if(v>=0)ShowCurs(v);
		cout << "\33[" << (wy=y) << ';' << (wx=x) << 'H';
		Flush();
		return *this;
	}
	inline Video &Goto(const int y,const int x,const int v,int v1,int v2)
	{
		cout << "\33[?" << (v1 ? 2 : 6) << 'c';		
		v1=v1, v2=v2;
		return Goto(y,x,v);
	}
}video;

static int Pt[5], Cptt[5];
typedef struct
{
	unsigned w, x[5];
}pala;
static class Alue
{
private:
	pala dummy, clear;
	vector<pala>field;
public:
	unsigned xdim, ydim;
public:
	Alue() : xdim(0), ydim(0)
	{
 		memset(&clear, 0, sizeof clear);
 		dummy = clear;
		dummy.w = 127;
	}
	void Resize(unsigned x,unsigned y)
	{
		xdim=x, ydim=y;
		field.resize(size());
		Clear();
	}
	inline unsigned size() const
	{
		return xdim*ydim;
	}
	void Clear()
	{
		unsigned x, y;
		for(y=0; y<ydim; y++)
			for(x=0; x<xdim; x++)
				field[x+y*xdim] = clear;
	}
	pala &ref(const int X,const int Y)
	{
		if(X<0||Y<0)return dummy;
		register const unsigned x=X, y=Y;
		if(x>=xdim||y>=ydim)return dummy;
		return field[x+y*xdim];
	}
} Alue;
#define Alue(x,y) Alue.ref(x,y)

class Kello
{
private:
	typedef unsigned char uc;
	unsigned x,y,xs,ys;
	uc *buf, *ca;
	float osa;
public:
	Kello(unsigned X,unsigned Y, unsigned XS,unsigned YS)
	:	x(X),y(Y),xs(XS),ys(YS), osa(-1)
	{
		if(y<12)y=12;
		if(y+ys>=(unsigned)LINES)ys=LINES-y-1;
		buf = (uc *)malloc(xs*ys*3);
		ca  = (uc *)malloc(xs*ys);
		Clear();
	}
	void Clear()
	{
		if(ca)memset(ca, 0, xs*ys);
	}
	~Kello()
	{
		if(buf)free(buf);
		if(ca) free(ca);
	}
	void PSet(const unsigned X,const unsigned Y)
	{
		if(buf)buf[(X/2)+Y*xs] |= 1 << (X&1);
	}
	void Line(const unsigned x1,const unsigned y1,
			  const unsigned x2,const unsigned y2)
	{
		unsigned NumPixels, i;
		const unsigned DeltaX = abs((int)x2 - (int)x1);
		const unsigned DeltaY = abs((int)y2 - (int)y1);
		int	d, dinc1, dinc2;
		int x, xinc1, xinc2;
		int y, yinc1, yinc2;
		if(DeltaX >= DeltaY)
		{
			NumPixels = DeltaX + 1;
			d = DeltaY * 2 - DeltaX;
   		 	dinc1 = DeltaY * 2;
			dinc2 = ((int)DeltaY - (int)DeltaX) * 2;
			yinc1 = 0, xinc1 = xinc2 = yinc2 = 1;
		}
		else
		{
			NumPixels = DeltaY + 1;
			d = DeltaX * 2 - DeltaY;
	   	 	dinc1 = DeltaX * 2;
			dinc2 = ((int)DeltaX - (int)DeltaY) * 2;
			xinc1 = 0, yinc1 = yinc2 = xinc2 = 1;
		}
		if(x1 > x2) {xinc1 = -xinc1; xinc2 = -xinc2;}
		if(y1 > y2) {yinc1 = -yinc1; yinc2 = -yinc2;}
		
		for(x=x1, y=y1, i=1; i<NumPixels; i++)
		{
			PSet(x, y);
			if(d<0)	d += dinc1, x += xinc1, y += yinc1;
			else	d += dinc2, x += xinc2, y += yinc2;
		}
	}
	void Viisari(const float p60, const float d1,const float d2)
	{
		register float rad = p60 * (3.141592653 / 30.0);
		Line(unsigned(xs+xs*d1*sin(rad)), unsigned(ys-ys*d1*cos(rad))*3/2,
			 unsigned(xs+xs*d2*sin(rad)), unsigned(ys-ys*d2*cos(rad))*3/2);
	}
	void Update()
	{
		if(!buf || !ca)return;
		time_t t = time(NULL);
		const struct tm *TM = localtime(&t);
		
		float sa = TM->tm_sec;
		
		if(sa == osa)return;
		
		osa = sa;
		
		memset(buf, 0, xs*ys*3);
		for(int a=0; a<60; a++)Viisari(a, a%5 ? 0.9 : 0.83, 1);
		
		float ma = TM->tm_min + sa / 60.0;
		float ha = TM->tm_hour*5.0 + ma / 60.0;
		
		Viisari(sa, 0,0.8);
		Viisari(ma, 0,0.7);
		Viisari(ha, 0,0.5);
		
		uc *s = buf;
		uc *e = ca;
		
		video = 0x10;
		
		for(unsigned Y=0; Y<ys; Y++, s += xs*2)
			for(unsigned X=0; X<xs; X++, e++, s++)
			{
        		/*  ab                                  */
        		/*  cd                       d          */
        		/*  ef         c      d    ddc          */
        		/*      b    ccb    ddb   dccb          */
        		/*    aba   caba   daba   caba          */
        		static const unsigned char Buf[] =
        			" `~^" "-''^" "-``^" "-`'\""  /*    */
        			".;;=" ",|/F" ",>/?" "r}/P"   /* e  */
        			".::="",\\<t"",\\|7""q\\{9"   /* f  */
        			"_===" "_LJC" ",jJ]" "wbd@";  /* ef */
        			
        		/*	ABCDEFGHIJKLMNOPQRSTUVWXYZ */
        		/*  abcdefghijklmnopqrstuvwxyz */
        		/*   !"#$%&'()*+,-./0123456789 */
        		/*  :;<=>?@[\]^_{|}~           */
        		
        		uc ch = Buf[ s[0] | (s[xs]<<2) | (s[xs*2]<<4) ];
        		
        		if(ch != *e)
        		{
        			video.Goto(y+Y, x+X, 0);
        			video << (*e=ch);
        		}
			}
		video.Flush();
	}
};
static void Center(const int y, const string &t)
{
	video.Goto(y, (COLS + 1 - t.size()) / 2) << t;
}

static int SubChecker(const int x2,const int y2,const unsigned whom,
					  const int xl,const int yl,const char mrk,
					  const unsigned me)
{
	/* Checks single direction */
	int pit=0, x, y, z;
	unsigned l1=5, l2=5;
	for(z=0,x=x2,y=y2; x+=xl,y+=yl,z<5; z++, pit++)
		if((l1=Alue(x,y).w)!=whom+1)break;
	if(z==5)goto won;
	if(l1)l1=0;else while((++l1,x+=xl,y+=yl,!Alue(x,y).w)&&l1<6);
	for(z=0,x=x2,y=y2; x-=xl,y-=yl,z<5; z++, pit++)	
		if((l2=Alue(x,y).w)!=whom+1)break;
	if(z==5)
	{
won:	for(x=x2,y=y2;;Alue(x,y).w=mrk){x+=xl,y+=yl;if(Alue(x,y).w != whom+1)break;}
		for(x=x2,y=y2;;Alue(x,y).w=mrk){x-=xl,y-=yl;if(Alue(x,y).w != whom+1)break;}
		return voitto;
	}
	if(pit==4)return (voitto-1); /* melkein voitto */
	if(l2)l2=0;else while((++l2,x-=xl,y-=yl,!Alue(x,y).w)&&l2<6);
	
	int space = l1+l2+1;
	if(space+pit>5)space=5-pit;
	return (space + pit >= 5 && pit)
	?
		-(20*pit*pit*pit + 2*space + 10*(l1&&l2) + (whom==me))
	:
		0;
}

static unsigned CheckPoint(const int x2,const int y2,unsigned CheckWhoPlayer,
                           int Joel,int Joel2)
{
	/* Except range errors! */
	// Parameters:
	//    (x2,y2)        = the point to check
	//    CheckWhoPlayer = whose status are we checking
	//    Joel2          = whose turn is it
	
	// Unused:
	Joel=Joel;		//   = computerplayer intelligency
	
	int l=0, k;
	if((k=SubChecker(x2,y2,CheckWhoPlayer, 1,0, '-',Joel2)) > 0)return k;l-=k;
	if((k=SubChecker(x2,y2,CheckWhoPlayer, 0,1, '|',Joel2)) > 0)return k;l-=k;
	if((k=SubChecker(x2,y2,CheckWhoPlayer, 1,1,'\\',Joel2)) > 0)return k;l-=k;
	if((k=SubChecker(x2,y2,CheckWhoPlayer,-1,1, '/',Joel2)) > 0)return k;l-=k;
	
	return l;
}

static void GetNiceInfo()
{
	unsigned x;
	video = 0x4F;
	video.clear();
	for(x=0; x<63; x++)
	{
		video = 0x40 + 7 + (x/32) * 8;
		video.Goto(10 + int(8 * sin(x / 5.0)), 40 + int(20 * cos(x / 5.0))) << 'X';
	}
	string d(
		"                                       "
		"* XOZGE2 is a gomoku clone for five players, made by Joel Yliluoma "
		"21-1-1994, for and with QuickBasic.                         "
		"In the strange years 1996-1999, the game was lost.           "
		"The source code was totally unlocatable, and "
		"Joel supposed it does not exist anymore.                     "
		"Then, 1.10.1999, he found it from an old file archive, "
		"remembered the whole thing and decided to port the game to "
		"Linux and C++.        This is it, the C++-version.      "
		"It seems to work. It took about ten hours to convert it. "
		"Have joy with this freeware game, which has been originally "
		"written by a 15 years old kid! :)                           "
		"Joel Yliluoma is also known as Bisqwit, in the Internet.    "
		"He has there a homepage, at http://iki.fi/bisqwit/ - watch it "
		"when you wish. * ");
	//   0123456789012345678901234567890123456789
	x = 2;
	while(!video.getkey())
	{
		usleep(100000);
		video = 0x15 + ((x % 4) * 2);
		if(x > 38)
			video.Goto(10,22) << d.substr(x, 38);
		else
			video.Goto(10,60-x) << d.substr(39, x);
		if(++x == d.length()+1)x = 2;
	}
	while(!Quittah&&video.getkey());
}

static void Help()
{
	video = 0x17;
	Center(1, "Some help of this program ...");
    Center(3, "When you run this program from DOS:");
    Center(4, "Usage is XOZGE2 [/Switches]. Shitwes may be /?, then you'll get more help.");
    Center(6, "When you start XOZGE2 without switches or from some sort of MENUprograms,");
    Center(7, "you should first come to crossboard selection.  There are also instructions,");
    Center(8, "so the crossboard is now undocumented in this help system.");
    Center(10, "When the selections are done, game starts.");
    Center(11, "The mission is to get five marks in the same line (before other players).");
    Center(14, "There are two type of players:");
    Center(15, "Humans:     Controlled from keyboard   ");
    Center(16, "Computers:  Controlled automatically,  ");
    Center(17, "              intelligencequotient can ");
    Center(18, "              be selected by user when ");
    Center(19, "              starting.  (Intelligence-");
    Center(20, "              quotient is a number;    ");
    Center(21, "              smaller number, computer ");
    Center(22, "              does more mistakes.)     ");
    Center(25, "Hit any key to return, you have read this text...");
    video.clear();
}

static void Help2()
{
	int x;
	for(x=0; x<20; x++) video << '\n';
	
	video.Goto(video.wherey()-18, 1);
	
	video = 0x3B; video << Str(COLS, 219) + "\n";
	video = 0x39; video << Str(COLS, 219) + "\n";
	video = 0x33; video << Str(COLS, 219) + "\n";
	video = 0x17; video << "* XOZGE = NEXT GENERATION OF GOMOKU ****** (        LINUX TRANSLATION        ) *\n";
	int Chooce = 1;
    video = 0x17; video << "" + Str(78, 205) + "\n";
    for(x=0; x<8; x++)
    	video <<           "" + Str(78, 32)  + "\n";
                  video << "" + Str(78, 205) + "\n";
    video.Goto(video.wherey()-9, 3) << "Usage :  xozge2 [switches]\n";
    video.Goto(video.wherey(), 4) << "      --help   This help screen\n";
    video.Goto(video.wherey(), 4) << "       -s1     Sounds when checking area are on\n";
    video.Goto(video.wherey(), 4) << "       -s2     Win- and losesounds are off\n";
    video.Goto(video.wherey(), 4) << "      --info   Get some nice information of this game's background\n";
	while(!Quittah)
    {
    	video = 0x0F + Chooce*8;
    	video.Goto(video.wherey()-3, 70) << ",------.\n";
    	video.Goto(video.wherey(), 70)   << "|  OK  |\n";
    	video.Goto(video.wherey(), 70)   << "`------'\n";
    	video = 0x0F + (3-Chooce)*8;
    	video.Goto(video.wherey()-6, 70) << ",------.\n";
    	video.Goto(video.wherey(), 70)   << "| HELP |\n";
    	video.Goto(video.wherey(), 70)   << "`------'\n\n\n\n";
    	while(!Quittah)
    	{
    		int i = video.getkey();
    		if(i=='\t'){Chooce = 3-Chooce;break;}
    		if(i=='\n'||i=='\r'||i==' ')
    		{
    			if(Chooce==2)return;
    			Help();
    			/* SOUND 850, 1: SOUND 705, 1 */
    }	}	}
}

static int MultiPlayers()
{
	string Ter[4];
	int x, Rivi, Posi;
	video = 0x17;
	video << Str(COLS, ' ');
	video.Goto(4, 18) << "Welcome!! Insert players to crossboard.\n";
	video = 13; video.Goto(24, 1) << "PS:  This is a QuickBasic -> C++ -translation for Linux, written 1.10.1999\n";
	video = 12; video.Goto(25, 1) << "     Copyright (C) 1992,1999 Bisqwit (http://iki.fi/bisqwit/)\n";
	video = 0x1F; video.Goto(18, 79) <<                                           "<";
	video.Goto(19, COLS) <<                                                        '';
	video.Goto(20, 23) << "Some help; what do you need to insert to highlighted box";
	video = 8;
	for(x=0; x<5; Cptt[x++]--);	/* 1000->999, 1->0 */
	for(x=0; x<2; x++)
	{
		video.Goto(10, 1+x*69) << " ";
		video.Goto(11, 1+x*69) << "  ۲";
		video.Goto(12, 1+x*69) << "  ۲";
		video.Goto(13, 1+x*69) << "  ۲";
		video.Goto(14, 1+x*69) << "  ۲";
		video.Goto(15, 1+x*69) << "  ۲";
		video.Goto(16, 1+x*69) << "  ۲";
		video.Goto(17, 1+x*69) << "";
	}
	video = 0x47;
	video.Goto(5, 20) << " The keys are described in there ";
	video.Goto(6, 20) << "             down.               ";
	video.Goto(7, 20) << "  ͻ  ";
	video.Goto(8, 20) << "͹ 1                      ͼ";
	video.Goto(9, 23) <<    "͹";
	video.Goto(10,23) <<    " 2                      ";
	video.Goto(11,23) <<    "͹";
	video.Goto(12,23) <<    " 3                      ";
	video.Goto(13,23) <<    "͹";
	video.Goto(14,23) <<    " 4                      ";
	video.Goto(15,23) <<    "͹";
	video.Goto(16,23) <<    " 5                      ";
	video.Goto(17,23) <<    "ͼ";
	for(x=0; x<5; x++)
	{
		video.Goto(8+x*2, 24);
		video = Attrs[x];
		video << XOZGE[x];
	}
	Rivi=0, Posi=!Quittah;
	do {
		int i = video.getkey();
		if(i=='H'||i=='A'||i=='w'||i=='i'||i=='')Rivi--;
		if(i=='P'||i=='B'||i=='s'||i=='k'||i=='')Rivi++;
		if(i=='K'||i=='D'||i=='a'||i=='j'||i==''
		|| i=='M'||i=='C'||i=='d'||i=='l'||i=='')Posi=3-Posi;
		if(isdigit(i)&&Posi==2&&Cptt[Rivi]<1000)Cptt[Rivi]=Cptt[Rivi]%100*10+i-'0';
		if(i=='\b'&&Posi==2)Cptt[Rivi]/=10;
		if((i=='\t'||i==' ')&&Posi==1)Pt[Rivi]++;
		if(Rivi==5)Rivi=0;else if(Rivi<0)Rivi=4;
		if(Posi==3)Posi=1;
		if(Pt[Rivi]==-1)Pt[Rivi]=2;
		if(Pt[Rivi]==3)Pt[Rivi]=0;
		if(i=='\n'||i=='\r')break;
		
		int x;
		for(x=0; x<5; x++)
		{
			if(Rivi==x)
			{
				if(Pt[x]==0)video = 0x1E-Random(6), video.Goto(19,1)<<"empty slot, no player here.                                                  ";
				if(Pt[x]==1)video = 0x1E-Random(6), video.Goto(19,1)<<"This player is a human, world's slaver...  Computer beats it any time!!        ";
				if(Pt[x]==2)video = 0x1E-Random(6), video.Goto(19,1)<<"This player is computer, $%%/( you can select it's intelligence!!          ";
			}
			video = 0x47;
			if(Rivi==x && Posi==1)video=0x17, video.Goto(18,1)<<"*** Use the TAB key to toggle between player types ***                        ",video=0x17;
			video.Goto(8+x*2, 29);
			if(Pt[x]==0)video << "None             ";
			if(Pt[x]==1)video << "Human            ";
			if(Pt[x]==2)video << "Computer         ";
			video = 0x47;
			if(Rivi==x && Posi==2)
			{
				video = 0x17;
				video.Goto(18, 1);
				if(Pt[x]==2)
					video << "** Insert to this box computerplayer's intelligencequotient **                ";
				else
					video << "    *** Humans intelligencequotient no needed to write here. ***              ";
			}
			video.Goto(8+x*2, 48);
			if(Pt[x]==2)
			{
				char Buf[5]; // Ensure
				sprintf(Buf, "%-3d", Cptt[x]);
				video << Buf;
			}
			else
				video << "---";
		}
	} while(!Quittah);
	int Prs = 5;
	for(Rivi=0; Rivi<5; Rivi++)
	{
		Cptt[Rivi]++; /* 999->1000, 0->1 */
		Prs -= !Pt[Rivi];
	}
	if(Prs<2)return 1;
	video = 0x17; video.clear();
	return 0;
}

static void TeeAlue(int xmul, int ymul)
{
	video = 7;
	
	Alue.Resize((COLS-22) / xmul, (LINES-2) / ymul);
	
	for(unsigned y=0; y<Alue.ydim; y++)
	{
		video.Goto(2 + y*ymul, 1) << (char)219;
		for(unsigned x=0; x<Alue.xdim; x++)
		{
			video << (char)250;
			for(int c=1; c<xmul; c++)video << ' ';
		}
		video << (char)219;
		for(int c=1; c<ymul; c++)
		{
			video.Goto(2+c + y*xmul, 1) << (char)219;
			video.Goto(video.wherey(), 1+xmul*Alue.xdim) << (char)219;
		}
	}
	video = 0x17;
	video.Goto(1,  1) << Str(Alue.xdim*xmul+2, 220);
	video.Goto(LINES, 1) << Str(Alue.xdim*xmul+2, (char)223);
}

static int Main()
{
	video = 7;
	video.clear();
	
	srand(time(NULL));
	
	video.Goto(1,1, 0,0,0);
	
	video = 0x70; video << "           X O Z G E   -   g a m e   w i t h   m a n y   p l a y e r s          \n";
	video = 0x74; video << "                A l l   i s   d o n e   w i t h o u t   p a p e r               \n";
	video = 0x78; video << "                                                                                \n";
	video = 7;
	
	/* Kun nappia DEL painetaan, tm kutsutaan ja sitten takas sama ruutu
	Help();
	 * Tekemtt viel
	 */
	 
	for(unsigned x=0; x<5; Cptt[x++]=1000);
	
	if(MultiPlayers())return 0;
	
	video = 0x13;
	video.Goto(5,COLS-19) << "Keys:";
	video.Goto(6,COLS-19) << "  arrows: move";
	video.Goto(7,COLS-19) << "  enter: place ";
	video.Goto(8,COLS-19) << "  ctrl-c: end";
	
	unsigned Moves=0, Moves2=0;
	unsigned Vuoro=5, Voitto=0;
	
	unsigned xmul = (LINES < COLS/2) ? 2 : 1;
	unsigned ymul = (COLS < LINES/2) ? 2 : 1;
	
	TeeAlue(xmul,ymul);
	
	unsigned x = Alue.xdim/2;
	unsigned y = Alue.ydim/2;
	
	Kello Kello(COLS-18, (LINES-18/xmul)/2, 17, 17/xmul);
	
	while(!Quittah)
	{
		if(++Vuoro >= 5)
		{
			Vuoro = 0;
			Moves2++;
		}
		
		if(!Pt[Vuoro])continue;
		
		video = 0x17;
		video.Goto(2,COLS-19) << "Player " << 1+Vuoro << "'s turn,\n";
		
		video = Attrs[Vuoro] | 0x10;
		video.Goto(7,76) << XOZGE[Vuoro];
		
		video = 7; video.Goto(1, 1) << "Investigating...";
/*  IF INSTR(Comman$, "/S1") THEN Length = 10: PLAY "mbt255l" + STR$(Length) + "o2aaggf" + STR$(CINT(Length / 2)) + "ef>c<bga" + STR$(CINT(Length / 4)) */

		video = 0x17;
		video.Goto(3, COLS-19) << "and " << Moves2+1;
		switch(Moves2 % 10)
		{
			case 1: video << "st"; break;
			case 2: video << "nd"; break;
			case 3: video << "rd"; break;
			default: video << "th";
		}
		video << " round.\n";
		for(int x2=x-4; x2<=(int)x+4; x2++)
			for(int y2=y-4; y2<=(int)y+4; y2++)
				for(unsigned a=0; a<5; a++)
				{
					unsigned b = CheckPoint(x2,y2, a, Cptt[Vuoro], Vuoro);
					if(b==voitto)
					{
						Voitto=a;
						goto Exit_Do;
					}
					Alue(x2,y2).x[a] = b;
				}
		video = 0x17;
		video.Goto(1, COLS-19) << Moves << " marks in area,";
		video.Goto(1, 1) << Str(19, 220);
		
		switch(Pt[Vuoro])
		{
			case 1:	/* Ihminen */
				for(int ox=-1,oy=-1, bx=x, by=y; !Quittah; )
				{
					bool brk=false;
					
					if(ox!=(int)x || oy!=(int)y)
					{
						ox=x, oy=y;
						video.Goto(LINES-1, COLS-15, 0);
						video = 0x13;
						video << Alue(x,y).x[0] << ':'
							  << Alue(x,y).x[1] << ':'
							  << Alue(x,y).x[2] << ':'
							  << Alue(x,y).x[3] << ':'
							  << Alue(x,y).x[4] << "      ";
					}
					
					video.Goto(2+y*ymul, 2+x*xmul, 1,0,15);
					Kello.Update();
					
					switch(video.getkey())
					{
						case '4': case 'K': case 'D':
						case 'a': case 'j': case '':
							if(x > 0)x--;
							break;
						case '6': case 'M': case 'C':
						case 'd': case 'l': case '':
							if(x < Alue.xdim-1)x++;
							break;
						case '8': case 'H': case 'A':
						case 'w': case 'i': case '':
							if(y > 0)y--;
							break;
						case '2': case 'P': case 'B':
						case 's': case 'k': case '':
							if(y < Alue.ydim-1)y++;
							break;
						case '\b':
							x=bx, y=by;
							break;
						case ' ': case '\r': case '\n':
							if(!Alue(x,y).w){brk=true; break;}
							video = 0x1C;
							video.Goto(4, COLS-16) << "Not here!!";
							video.Flush();
							sleep(1);
							video.Goto(4, COLS-16) << "          ";
					}
					if(brk)break;
				}
				break;
			case 2:
			{
				vector< pair<unsigned,unsigned> > Eni;
				video = 7;
				video.Goto(1,1, 0) << "Player " << Vuoro << " thinking its move";
				video = 0x87;
				video << ". . .";
			GetPla:
				unsigned z=0;
				int BZ=-1;
				Kello.Update();
				for(unsigned x2=0; x2<Alue.xdim; x2++)
					for(unsigned y2=0; y2<Alue.ydim; y2++)
					{
						if(Alue(x2,y2).w){z++;continue;}
						if(rnd() < Cptt[Vuoro] / 1000.0)
						{
							int a, z2;
							for(a=z2=0; a<5; a++)
								z2 += Alue(x2,y2).x[a];
								
							/* Eni is a list of equivalent locations
							 *
							 * Now if we get a match, we add this to the
							 * list. If we get better, we clear the list
							 * and put this to the bottom.
							 */
							 
							// (x2,y2) = coordinates
							// z2      = interestingness value
								
							if(z2 < BZ)continue;
							if(z2 > BZ)
							{
								Eni.clear();
								BZ = z2;
							}
							
							Eni.insert(Eni.end(),
								pair<unsigned,unsigned>(x2,y2));
						}
					}
					
				/* TODO: Move this away from here
				 * Let the player have a chance to
				 * win with the last hit
				 */
				if(z == Alue.size())
				{
					video = 0x1E;
					video.Goto(10, 4) << "ͻ";
					video.Goto(11, 4) << " LET'S CLEAR THE AREA NOW ... ";
					video.Goto(12, 4) << "ͼ";
					video.Flush();
					sleep(1);
					TeeAlue(xmul,ymul);
					goto GetPla;
				}
				
				int z2 = Eni.size() ? 0 : 1000;
				#if 0
			Test:
				#endif
				do {
					if(z2++ > 1000)
					{
						x = Random(Alue.xdim);
						y = Random(Alue.ydim);
					}
					else
					{
						z = Random(Eni.size());
						x = Eni[z].first;
						y = Eni[z].second;
					}
				} while(Alue(x,y).w);
				#if 0 // label not used blah blah..
			Vihdoinkin:
				#endif
				video = 0x17;
				video.Goto(1, 1) << Str((Alue.xdim+2)*xmul, 220);
				break;
			}
		}
		video.Goto(2+y*ymul, 2+x*xmul, 0,0,15);
		
		video = Attrs[Vuoro];
		video << XOZGE[Vuoro];
		
		video.Goto(2+y*ymul, 2+x*xmul, 1,0,15);
		
		Alue(x,y).w = Vuoro+1;
		Moves++;
	}
Exit_Do:;
/*
IF INSTR(Comman$, "/S2") = 0 THEN
    IF Pt(Voitto) = 1 THEN PLAY "mbt255o3l4ceg>c<geceg>c2"
    IF Pt(Voitto) = 2 THEN
        FOR x = 1 TO 3
            PLAY "mbt255o3l16fp4fp4eo1l64ce-"
            PLAY "gl16o3p4fp4>cp2<"
            PLAY "o1l64ce-gl16o3fp4ep4"
        NEXT
        PLAY "mbt255o3l16>c<bp8a+ap8g+gp8f+fp8ed+dp8c+c"
        PLAY "mbo0l32cfegdfcgefdcg"
    END IF
END IF
*/
	video.Goto(1,COLS-19,1,13,15) << "Pelaaja " << Voitto+1 << " voitti.\n";
	video.Goto(LINES,1) << "*";
	video.Goto(LINES,COLS)<< "*";
	video.Goto(LINES-1,1) << "Pelataanko uudestaan (K / E) [K]\b\b";
	string d(" * XOZGE2 - Five-players gomoku was made by Joel Yliluoma 21-1-1994 - this is the Linux-port version 1.00 * ");
	unsigned char ke = Quittah?'E':'K';
	for(x=2;!Quittah;)
	{
		unsigned char a=toupper(video.getkey());
		usleep(40000);
		if(a=='E' || a=='K') { video << (ke=a) << '\b'; }
		video = 5 + (x % 4) * 2;
		int y2 = video.wherey();
		int x2 = video.wherex();
		if(x < d.size()+1)
		{
			video.Goto(LINES, 2) << d.substr(d.size()-x, x);
		}
		else
		{
			if(x == d.size()+1)x += 2;
			video.Goto(LINES, x - d.size()) << d.substr(0, COLS - x + d.size());
		}
		if(++x > d.size()+COLS)x = 2;
		time_t z = time(NULL);
		unsigned Z = localtime(&z)->tm_sec;
		if(Z!=y)
		{
			unsigned x;
			video = Attrs[Voitto];
			for(y=0; y<Alue.ydim; y++)
				for(x=0; x<Alue.xdim; x++)
					if(Alue(x,y).w > 5)
					{
						video.Goto(2+y*ymul, 2+x*xmul, 0) << (char)
									(Z%2 ? XOZGE[Voitto] : Alue(x,y).w);
					}
			y=Z;
		}
		video.Goto(y2, x2, 1);
		if(a=='\n')break;
	}
	return ke != 'E';
}

static void CtrlC(int d)
{
	d=d;
	Quittah++;
}
int main(int argc, const char **argv)
{
	signal(SIGINT, CtrlC);
	signal(SIGQUIT, CtrlC);
	
	while(--argc)
	{
		const char *s = *++argv;
		if(string(s) == "--help") { Help2(); return 0; }
		if(string(s) == "--info") GetNiceInfo();
	}
	while(Main());
	return 0;
}
